package com.jms.queue.async.mdb;
/**
 * The MDBTextListener class implements the MessageListener 
 * interface by defining an onMessage method that displays 
 * the contents of a TextMessage.
 *
 * This class acts as the message driven bean listener.
 */
import javax.annotation.Resource;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;

import org.jboss.ejb3.annotation.ResourceAdapter;

@MessageDriven(activationConfig = {
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
		@ActivationConfigProperty(propertyName = "destination", propertyValue = "testQueue"),
		@ActivationConfigProperty(propertyName = "user", propertyValue = "srinivaas"),
		@ActivationConfigProperty(propertyName = "password", propertyValue = "password"),
		@ActivationConfigProperty(propertyName = "maxSession", propertyValue = MDBTextListener.poolSize)})
@ResourceAdapter("remote-hornetq-ra.rar")

@TransactionManagement(value = TransactionManagementType.CONTAINER)
@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
public class MDBTextListener implements MessageListener {
	public static final String poolSize = "10";
	
	@Resource(mappedName = "java:/RemoteJmsXA")
	ConnectionFactory connectionFactory;
	
	@Resource(mappedName = "java:/jboss/exported/jms/queue/responseQ")
	Queue replyQueue;
	
	private volatile Connection connection;
	private volatile Session session;
	private volatile MessageProducer producer;

	   
    /**
	 * Casts the message to a TextMessage and displays its text.
	 * 
	 * @param message
	 *            the incoming message
	 */
    public void onMessage(Message message) {
        TextMessage msg = null;
        String text="";    
        try {
            if (message instanceof TextMessage) {
                msg = (TextMessage) message;
                text = msg.getText();
                System.out.println("MDB Text Listener Completed Successfully...");
            } else {
                System.out.println("Message of wrong type: " + message.getClass().getName());
            }
        } catch (JMSException e) {
            System.out.println("JMSException in onMessage(): " +
                e.toString());
        } catch (Throwable t) {
            System.out.println("Exception in onMessage():" +
                t.getMessage());
        }
        
        sendResponse(text);
    }
    
    public void sendResponse(String text) {
		try {
			
			// Step 11. we create a JMS connection
			connection = connectionFactory.createConnection();
			
			// Step 12. We create a JMS session
			session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

			// Step 13. we create a producer for the reply queue
			producer = session.createProducer(replyQueue);

			// Step 14. we create a message and send it
			producer.send(session.createTextMessage("reply for " + text));

			System.out.println("Sent response message " + text);
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			if (connection != null) {
				try {
					connection.close();
				} catch (JMSException e) {
				}
			}
		}
	}
}